/**
 * Copyright (C) 2009 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef WK_KEY_FILE_H
#define WK_KEY_FILE_H

#include <glib-object.h>

G_BEGIN_DECLS

/*
 * ConfigItem
 */
typedef enum
{
    CONFIG_ITEM_TYPE_INT = 0,
    CONFIG_ITEM_TYPE_BOOLEAN,
    CONFIG_ITEM_TYPE_STRING,
    CONFIG_ITEM_TYPE_STRING_FROM_URI,
    CONFIG_ITEM_TYPE_INVALID
} ConfigItemType;

typedef struct _ConfigItem ConfigItem;

ConfigItem *wk_config_item_new (const gchar    *section,
                                const gchar    *key,
                                const gchar    *uri,
                                ConfigItemType  type,
                                gpointer        value);

ConfigItemType wk_config_item_get_type (ConfigItem *item);

void     wk_config_item_set_value  (ConfigItem   *item,
                                    gpointer      value,
                                    gboolean      free_old);
gpointer wk_config_item_get_value  (ConfigItem   *item);
void     wk_config_item_get        (ConfigItem   *item,
                                    const gchar **section,
                                    const gchar **key,
                                    const gchar **uri,
                                    guint        *type,
                                    gpointer     *value);

void wk_config_item_free (ConfigItem     *item);


/*
 * WkKeyFile
 */
typedef struct _WkKeyFilePrivate WkKeyFilePrivate;
typedef struct _WkKeyFileClass   WkKeyFileClass;
typedef struct _WkKeyFile        WkKeyFile;

#define WK_TYPE_KEY_FILE             (wk_key_file_get_type ())
#define WK_KEY_FILE(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), WK_TYPE_KEY_FILE, WkKeyFile))
#define WK_KEY_FILE_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), WK_TYPE_KEY_FILE, WkKeyFileClass))
#define WK_IS_KEY_FILE(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WK_TYPE_KEY_FILE))
#define WK_IS_KEY_FILE_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), WK_TYPE_KEY_FILE))
#define WK_KEY_FILE_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), WK_TYPE_KEY_FILE, WkKeyFileClass))

struct _WkKeyFileClass
{
    GObjectClass __parent__;
};

struct _WkKeyFile
{
    GObject __parent__;

    WkKeyFilePrivate *priv;
};

GType wk_key_file_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;

WkKeyFile *wk_key_file_new   (void);

gboolean   wk_key_file_load_from_data (WkKeyFile *key_file,
                                       const gchar *data);
gboolean   wk_key_file_load_from_file (WkKeyFile *key_file,
                                       const gchar *filename);
gboolean   wk_key_file_load_from_uri  (WkKeyFile *key_file,
                                       const gchar *uri);

gchar **wk_key_file_get_keys     (WkKeyFile   *key_file,
                                  const gchar *section,
                                  gsize       *length,
                                  GError     **error);
gchar **wk_key_file_get_sections (WkKeyFile *key_file,
                                  gsize     *length);

ConfigItem *wk_key_file_get_config_item (WkKeyFile *key_file,
                                         const gchar *section,
                                         const gchar *key,
                                         ConfigItemType type,
                                         gpointer fallback);
void        wk_key_file_set_config_item (WkKeyFile *key_file,
                                         ConfigItem *item);

gint     wk_key_file_get_int             (WkKeyFile   *key_file,
                                          const gchar *section,
                                          const gchar *key,
                                          int          fallback);
gboolean wk_key_file_get_boolean         (WkKeyFile   *key_file,
                                          const gchar *section,
                                          const gchar *key,
                                          gboolean     fallback);
gchar   *wk_key_file_get_string          (WkKeyFile  *key_file,
                                          const gchar *section,
                                          const gchar *key,
                                          const gchar *fallback);
gchar   *wk_key_file_get_string_from_uri (WkKeyFile   *key_file,
                                          const gchar *section,
                                          const gchar *key,
                                          gchar      **uri,
                                          const gchar *fallback);

gchar   *wk_key_file_get_data (WkKeyFile *key_file);
gboolean wk_key_file_write    (WkKeyFile   *key_file,
                               const gchar *filename);

G_END_DECLS

#endif /* !WK_KEY_FILE_H */
