/**
 * Copyright (C) 2009 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SETTINGS_H
#define SETTINGS_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib.h>

#include "wk-key-file.h"

typedef struct _ConfigItemDesc
{
    const gchar *section;
    const gchar *key;
    ConfigItemType type;

    /* this has to be modified, it sucks */
    gchar *fallback;
} ConfigItemDesc;

GHashTable *handled_settings_get_from_array (WkKeyFile *kf,
                                             ConfigItemDesc *descs);
void        handled_settings_set_from_hash  (WkKeyFile *kf,
                                             GHashTable *hash);

#define HANDLED_SETTINGS_GET_INT(hash,str) \
    (gint) wk_config_item_get_value ((ConfigItem *) g_hash_table_lookup ((hash), (str)))
#define HANDLED_SETTINGS_GET_BOOLEAN(hash,str) \
    (gboolean) wk_config_item_get_value ((ConfigItem *) g_hash_table_lookup ((hash), (str)))
#define HANDLED_SETTINGS_GET_STRING(hash,str) \
    (gchar *) wk_config_item_get_value ((ConfigItem *) g_hash_table_lookup ((hash), (str)))

#define HANDLED_SETTINGS_SET_INT(hash,str,value) \
    wk_config_item_set_value ((ConfigItem *) g_hash_table_lookup (\
        (hash), (str)), GINT_TO_POINTER ((value)), FALSE)
#define HANDLED_SETTINGS_SET_BOOLEAN(hash,str,value) \
    wk_config_item_set_value ((ConfigItem *) g_hash_table_lookup (\
        (hash), (str)), GINT_TO_POINTER ((gboolean) (value)), FALSE)
#define HANDLED_SETTINGS_SET_STRING(hash,str,value) \
    wk_config_item_set_value ((ConfigItem *) g_hash_table_lookup (\
        (hash), (str)), (gpointer) (value), TRUE)
#define HANDLED_SETTINGS_SET_STRING_DUP(hash,str,value) \
    wk_config_item_set_value ((ConfigItem *) g_hash_table_lookup (\
        (hash), (str)), (gpointer) (g_strdup (value)), TRUE)

#endif /* !SETTINGS_H */
